@@ -7,7 +7,9 @@
 
 #include "chatdocumenthandler.h"
 #include "eventhandler.h"
+#include "models/actionsmodel.h"
 #include "neochatroom.h"
+#include "texthandler.h"
 
 ChatBarCache::ChatBarCache(QObject *parent)
     : QObject(parent)
@@ -28,6 +30,37 @@ void ChatBarCache::setText(const QString &text)
     Q_EMIT textChanged();
 }
 
+QString ChatBarCache::sendText() const
+{
+    if (!attachmentPath().isEmpty()) {
+        QUrl url(attachmentPath());
+        auto path = url.isLocalFile() ? url.toLocalFile() : url.toString();
+        return text().isEmpty() ? path.mid(path.lastIndexOf(u'/') + 1) : text();
+    }
+
+    return formatMentions();
+}
+
+QString ChatBarCache::formatMentions() const
+{
+    auto mentions = m_mentions;
+    std::sort(mentions.begin(), mentions.end(), [](const auto &a, const auto &b) {
+        return a.cursor.anchor() > b.cursor.anchor();
+    });
+
+    auto formattedText = text();
+    for (const auto &mention : mentions) {
+        if (mention.text.isEmpty() || mention.id.isEmpty()) {
+            continue;
+        }
+        formattedText = formattedText.replace(mention.cursor.anchor(),
+                                              mention.cursor.position() - mention.cursor.anchor(),
+                                              QStringLiteral("[%1](https://matrix.to/#/%2)").arg(mention.text.toHtmlEscaped(), mention.id));
+    }
+
+    return formattedText;
+}
+
 bool ChatBarCache::isReplying() const
 {
     return m_relationType == Reply && !m_relationId.isEmpty();
@@ -53,6 +86,7 @@ void ChatBarCache::setReplyId(const QString &replyId)
         m_relationType = Reply;
     }
     m_attachmentPath = QString();
+    delete m_relationContentModel;
     Q_EMIT relationIdChanged(oldEventId, m_relationId);
     Q_EMIT attachmentPathChanged();
 }
@@ -82,11 +116,12 @@ void ChatBarCache::setEditId(const QString &editId)
         m_relationType = Edit;
     }
     m_attachmentPath = QString();
+    delete m_relationContentModel;
     Q_EMIT relationIdChanged(oldEventId, m_relationId);
     Q_EMIT attachmentPathChanged();
 }
 
-Quotient::RoomMember ChatBarCache::relationUser() const
+Quotient::RoomMember ChatBarCache::relationAuthor() const
 {
     if (parent() == nullptr) {
         qWarning() << "ChatBarCache created with no parent, a NeoChatRoom must be set as the parent on creation.";
@@ -119,16 +154,33 @@ QString ChatBarCache::relationMessage() const
     }
 
     if (auto event = room->findInTimeline(m_relationId); event != room->historyEdge()) {
-        EventHandler eventhandler(room, &**event);
-        if (isEditing()) {
-            return eventhandler.getMarkdownBody();
-        } else {
-            return eventhandler.getMarkdownBody().toHtmlEscaped();
-        }
+        return EventHandler::markdownBody(&**event);
     }
     return {};
 }
 
+MessageContentModel *ChatBarCache::relationEventContentModel()
+{
+    if (parent() == nullptr) {
+        qWarning() << "ChatBarCache created with no parent, a NeoChatRoom must be set as the parent on creation.";
+        return nullptr;
+    }
+    if (m_relationId.isEmpty()) {
+        return nullptr;
+    }
+    if (m_relationContentModel != nullptr) {
+        return m_relationContentModel;
+    }
+
+    auto room = dynamic_cast<NeoChatRoom *>(parent());
+    if (room == nullptr) {
+        qWarning() << "ChatBarCache created with incorrect parent, a NeoChatRoom must be set as the parent on creation.";
+        return nullptr;
+    }
+    m_relationContentModel = new MessageContentModel(room, m_relationId, true);
+    return m_relationContentModel;
+}
+
 bool ChatBarCache::isThreaded() const
 {
     return !m_threadId.isEmpty();
@@ -144,8 +196,8 @@ void ChatBarCache::setThreadId(const QString &threadId)
     if (m_threadId == threadId) {
         return;
     }
-    m_threadId = threadId;
-    Q_EMIT threadIdChanged();
+    const auto oldThreadId = std::exchange(m_threadId, threadId);
+    Q_EMIT threadIdChanged(oldThreadId, m_threadId);
 }
 
 QString ChatBarCache::attachmentPath() const
@@ -161,10 +213,22 @@ void ChatBarCache::setAttachmentPath(const QString &attachmentPath)
     m_attachmentPath = attachmentPath;
     m_relationType = None;
     const auto oldEventId = std::exchange(m_relationId, QString());
+    delete m_relationContentModel;
     Q_EMIT attachmentPathChanged();
     Q_EMIT relationIdChanged(oldEventId, m_relationId);
 }
 
+void ChatBarCache::clearRelations()
+{
+    const auto oldEventId = std::exchange(m_relationId, QString());
+    const auto oldThreadId = std::exchange(m_threadId, QString());
+    m_attachmentPath = QString();
+    delete m_relationContentModel;
+    Q_EMIT relationIdChanged(oldEventId, m_relationId);
+    Q_EMIT threadIdChanged(oldThreadId, m_threadId);
+    Q_EMIT attachmentPathChanged();
+}
+
 QList<Mention> *ChatBarCache::mentions()
 {
     return &m_mentions;
@@ -228,4 +292,43 @@ void ChatBarCache::setSavedText(const QString &savedText)
     m_savedText = savedText;
 }
 
+void ChatBarCache::postMessage()
+{
+    auto room = dynamic_cast<NeoChatRoom *>(parent());
+    if (room == nullptr) {
+        qWarning() << "ChatBarCache created with incorrect parent, a NeoChatRoom must be set as the parent on creation.";
+        return;
+    }
+
+    if (!attachmentPath().isEmpty()) {
+        room->uploadFile(QUrl(attachmentPath()), sendText());
+        clearCache();
+        return;
+    }
+
+    const auto result = ActionsModel::handleAction(room, this);
+    if (!result.first.has_value()) {
+        return;
+    }
+
+    TextHandler textHandler;
+    textHandler.setData(*std::get<std::optional<QString>>(result));
+    const auto sendText = textHandler.handleSendText();
+
+    if (sendText.length() == 0) {
+        return;
+    }
+
+    room->postMessage(text(), sendText, *std::get<std::optional<Quotient::RoomMessageEvent::MsgType>>(result), replyId(), editId(), threadId());
+    clearCache();
+}
+
+void ChatBarCache::clearCache()
+{
+    setText({});
+    m_mentions.clear();
+    m_savedText = QString();
+    clearRelations();
+}
+
 #include "moc_chatbarcache.cpp"
