@@ -7,6 +7,7 @@
 #include <QQmlEngine>
 
 #include "neochatconnection.h"
+#include "notificationsmanager.h"
 #include <Quotient/accountregistry.h>
 
 class TrayIcon;
@@ -53,16 +54,6 @@ class Controller : public QObject
     Q_PROPERTY(bool csSupported READ csSupported CONSTANT)
 
 public:
-    /**
-     * @brief Define the types on inline messages that can be shown.
-     */
-    enum MessageType {
-        Positive, /**< Positive message, typically green. */
-        Info, /**< Info message, typically highlight color. */
-        Error, /**< Error message, typically red. */
-    };
-    Q_ENUM(MessageType)
-
     static Controller &instance();
     static Controller *create(QQmlEngine *engine, QJSEngine *)
     {
@@ -98,6 +89,13 @@ public:
      */
     static void listenForNotifications();
 
+    /**
+     * @brief Clear an existing invite notification for the given room.
+     *
+     * Nothing happens if the given room doesn't have an invite notification.
+     */
+    Q_INVOKABLE void clearInvitationNotification(const QString &roomId);
+
     Q_INVOKABLE QString loadFileContent(const QString &path) const;
 
     Quotient::AccountRegistry &accounts();
@@ -108,6 +106,17 @@ public:
 
     bool csSupported() const;
 
+    /**
+     * @brief Revert all configuration values to their default.
+     *
+     * The parameters along with their defaults are specified in the config file
+     * neochatconfig.kcfg.
+     */
+    Q_INVOKABLE void revertToDefaultConfig();
+
+    Q_INVOKABLE bool isImageShown(const QString &eventId);
+    Q_INVOKABLE void markImageShown(const QString &eventId);
+
 private:
     explicit Controller(QObject *parent = nullptr);
 
@@ -116,13 +125,13 @@ private:
 
     QKeychain::ReadPasswordJob *loadAccessTokenFromKeyChain(const QString &account);
 
-    void loadSettings();
-    void saveSettings() const;
-
     Quotient::AccountRegistry m_accountRegistry;
     QStringList m_accountsLoading;
     QMap<QString, QPointer<NeoChatConnection>> m_connectionsLoading;
     QString m_endpoint;
+    QStringList m_shownImages;
+
+    NotificationsManager m_notificationsManager;
 
 private Q_SLOTS:
     void invokeLogin();
@@ -130,10 +139,12 @@ private Q_SLOTS:
     void updateBadgeNotificationCount(NeoChatConnection *connection, int count);
 
 Q_SIGNALS:
-    void errorOccured(const QString &error, const QString &detail);
+    /**
+     * @brief Request a error message be shown to the user.
+     */
+    void errorOccured(const QString &error);
     void connectionAdded(NeoChatConnection *connection);
     void connectionDropped(NeoChatConnection *connection);
     void activeConnectionChanged(NeoChatConnection *connection);
     void accountsLoadingChanged();
-    void showMessage(MessageType messageType, const QString &message);
 };
