@@ -6,17 +6,12 @@
 #include <QImageReader>
 #include <QJsonDocument>
 
-#include "controller.h"
 #include "jobs/neochatchangepasswordjob.h"
 #include "jobs/neochatdeactivateaccountjob.h"
-#include "linkpreviewer.h"
 #include "neochatconfig.h"
 #include "neochatroom.h"
-#include "notificationsmanager.h"
-#include "roommanager.h"
 #include "spacehierarchycache.h"
 
-#include <Quotient/connection.h>
 #include <Quotient/jobs/basejob.h>
 #include <Quotient/quotient_common.h>
 #include <qt6keychain/keychain.h>
@@ -46,6 +41,7 @@ NeoChatConnection::NeoChatConnection(QObject *parent)
     : Connection(parent)
     , m_threePIdModel(new ThreePIdModel(this))
 {
+    m_linkPreviewers.setMaxCost(20);
     connectSignals();
 }
 
@@ -53,6 +49,7 @@ NeoChatConnection::NeoChatConnection(const QUrl &server, QObject *parent)
     : Connection(server, parent)
     , m_threePIdModel(new ThreePIdModel(this))
 {
+    m_linkPreviewers.setMaxCost(20);
     connectSignals();
 }
 
@@ -68,10 +65,6 @@ void NeoChatConnection::connectSignals()
     });
     connect(this, &NeoChatConnection::syncDone, this, [this] {
         setIsOnline(true);
-
-        connect(this, &NeoChatConnection::syncDone, this, [this]() {
-            NotificationsManager::instance().handleNotifications(this);
-        });
     });
     connect(this, &NeoChatConnection::networkError, this, [this]() {
         setIsOnline(false);
@@ -81,9 +74,9 @@ void NeoChatConnection::connectSignals()
             Q_EMIT userConsentRequired(job->errorUrl());
         }
     });
-    connect(this, &NeoChatConnection::requestFailed, this, [](BaseJob *job) {
+    connect(this, &NeoChatConnection::requestFailed, this, [this](BaseJob *job) {
         if (dynamic_cast<DownloadFileJob *>(job) && job->jsonData()["errcode"_ls].toString() == "M_TOO_LARGE"_ls) {
-            RoomManager::instance().warning(i18n("File too large to download."), i18n("Contact your matrix server administrator for support."));
+            Q_EMIT showMessage(MessageType::Warning, i18n("File too large to download.<br />Contact your matrix server administrator for support."));
         }
     });
     connect(this, &NeoChatConnection::directChatsListChanged, this, [this](DirectChatsMap additions, DirectChatsMap removals) {
@@ -116,6 +109,10 @@ void NeoChatConnection::connectSignals()
             Q_EMIT homeHaveHighlightNotificationsChanged();
         });
     });
+    connect(this, &NeoChatConnection::invitedRoom, this, [this](Quotient::Room *room) {
+        auto r = dynamic_cast<NeoChatRoom *>(room);
+        connect(r, &NeoChatRoom::showInviteNotification, this, &NeoChatConnection::showInviteNotification);
+    });
     connect(this, &NeoChatConnection::leftRoom, this, [this](Room *room, Room *prev) {
         Q_UNUSED(room)
         if (prev && prev->isDirectChat()) {
@@ -344,17 +341,9 @@ void NeoChatConnection::createRoom(const QString &name, const QString &topic, co
             }
         });
     }
-    connect(job, &CreateRoomJob::failure, this, [job] {
-        Q_EMIT Controller::instance().errorOccured(i18n("Room creation failed: %1", job->errorString()), {});
+    connect(job, &CreateRoomJob::failure, this, [this, job] {
+        Q_EMIT errorOccured(i18n("Room creation failed: %1", job->errorString()));
     });
-    connect(
-        this,
-        &Connection::newRoom,
-        this,
-        [](Room *room) {
-            RoomManager::instance().resolveResource(room->id());
-        },
-        Qt::SingleShotConnection);
 }
 
 void NeoChatConnection::createSpace(const QString &name, const QString &topic, const QString &parent, bool setChildParent)
@@ -381,17 +370,9 @@ void NeoChatConnection::createSpace(const QString &name, const QString &topic, c
             }
         });
     }
-    connect(job, &CreateRoomJob::failure, this, [job] {
-        Q_EMIT Controller::instance().errorOccured(i18n("Space creation failed: %1", job->errorString()), {});
+    connect(job, &CreateRoomJob::failure, this, [this, job] {
+        Q_EMIT errorOccured(i18n("Space creation failed: %1", job->errorString()));
     });
-    connect(
-        this,
-        &Connection::newRoom,
-        this,
-        [](Room *room) {
-            RoomManager::instance().resolveResource(room->id());
-        },
-        Qt::SingleShotConnection);
 }
 
 bool NeoChatConnection::directChatExists(Quotient::User *user)
@@ -399,29 +380,6 @@ bool NeoChatConnection::directChatExists(Quotient::User *user)
     return directChats().contains(user);
 }
 
-void NeoChatConnection::openOrCreateDirectChat(const QString &userId)
-{
-    if (auto user = this->user(userId)) {
-        openOrCreateDirectChat(user);
-    } else {
-        qWarning() << "openOrCreateDirectChat: Couldn't get user object for ID " << userId << ", unable to open/request direct chat.";
-    }
-}
-
-void NeoChatConnection::openOrCreateDirectChat(User *user)
-{
-    const auto existing = directChats();
-
-    if (existing.contains(user)) {
-        const auto room = this->room(existing.value(user));
-        if (room) {
-            RoomManager::instance().resolveResource(room->id());
-            return;
-        }
-    }
-    requestDirectChat(user->id());
-}
-
 qsizetype NeoChatConnection::directChatNotifications() const
 {
     qsizetype notifications = 0;
@@ -518,6 +476,7 @@ QCoro::Task<void> NeoChatConnection::setupPushNotifications(QString endpoint)
         qWarning() << "There's no gateway, not setting up push notifications.";
     }
 #else
+    Q_UNUSED(endpoint)
     co_return;
 #endif
 }
@@ -564,14 +523,29 @@ LinkPreviewer *NeoChatConnection::previewerForLink(const QUrl &link)
         return nullptr;
     }
 
-    auto previewer = m_linkPreviewers.value(link, nullptr);
+    auto previewer = m_linkPreviewers.object(link);
     if (previewer != nullptr) {
         return previewer;
     }
 
     previewer = new LinkPreviewer(link, this);
-    m_linkPreviewers[link] = previewer;
+    m_linkPreviewers.insert(link, previewer);
     return previewer;
 }
 
+KeyImport::Error NeoChatConnection::exportMegolmSessions(const QString &passphrase, const QString &path)
+{
+    KeyImport keyImport;
+    auto result = keyImport.exportKeys(passphrase, this);
+    if (!result.has_value()) {
+        return result.error();
+    }
+    QUrl url(path);
+    QFile file(url.toLocalFile());
+    file.open(QFile::WriteOnly);
+    file.write(result.value());
+    file.close();
+    return KeyImport::Success;
+}
+
 #include "moc_neochatconnection.cpp"
